/*
********************************************************************
(c) MSU Video Group 2003-2006, http://compression.ru/video/
This source code is property of MSU Graphics and Media Lab

This code may be distributed under LGPL
(see http://www.gnu.org/licenses/lgpl.html for more details).

E-mail: video-measure@compression.ru
Author: Alexey Noskov
********************************************************************
*/  

/*
* bfm.cpp: implementation of VMT plugin for measurement of MSU Brightness Flicking Measure
*/

#include "stdafx.h"

#include "pluginadapter.h"
#include "bfm.h"

template < typename T > T sqr ( const T & a ) { return a * a; }

/*
* DllMain
*/

BOOL APIENTRY DllMain ( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
    return TRUE;
}

extern "C" __declspec(dllexport) void CreateMetric ( IMetricPlugin**metric )
{
	*metric = new CPluginAdapter ( new CBFMPlugin );
}

extern "C" __declspec(dllexport) void ReleaseMetric ( IMetricPlugin* metric )
{
	delete metric;
}

extern "C" __declspec(dllexport) int GetVQMTVersion ()
{
	return 302;
}

extern "C" __declspec(dllexport) int CompatibleWithVQMT(int vqmtVer)
{
	return vqmtVer >= 302 ? 0 : -1;
}

/*
* Construction/destruction
*/

CBFMPlugin::CBFMPlugin ()
{
}

CBFMPlugin::~CBFMPlugin ()
{
}

/*
* Initialization
*/

void CBFMPlugin::Init ( IMetricPlugin::COLORCOMPONENT cc, int width, int height, int &global_id )
{
	m_bfm = global_id++;

	m_width = width;
	m_height = height;
	m_size = width * height;

	m_frames_num = 0;

	m_b_prev = -1.0f;
	m_b_error = 0.0f;
}

/*
* Measurement
*/

vector< pair <IMetricPlugin::ID, float> > CBFMPlugin::Measure ( vector<IMetricImage*> &images )
{
	float * plane = images [ 0 ]->GetY ();

	float b_summ = 0;
	float b_error = 0;
	
	/*
	* Calculating average brightness
	*/
	for ( int i = 0; i < m_size; ++ i, ++ plane ) {
		b_summ += *plane;
	}

	b_summ /= m_size;
	
	/*
	* Calculating brightness error - difference with previous frame
	*/
	b_error  = abs ( b_summ - m_b_prev );

	if ( m_b_prev < 0 ) b_error = 0.0f;

	m_b_prev = b_summ;
	m_b_error += b_error;	
	m_frames_num ++;

	vector< pair <IMetricPlugin::ID, float> > res;

	res.push_back ( make_pair ( m_bfm, b_error ) );
	return res;
}

vector< pair <IMetricPlugin::ID, float> > CBFMPlugin::MeasureAndVisualize ( vector<IMetricImage*>&images, BYTE *vis, int vis_pitch )
{
	float * plane_cur = images [ 0 ]->GetY ();

	for ( int i = 0; i < m_height; ++i) 
    {
        for ( int j = 0; j < m_width; ++j) 
	    {
		    BYTE val = BYTE ( *plane_cur );
		    SetRGB ( vis + j * 3, 0, 0, val, val, val );
		    plane_cur++;
	    }
	    vis += vis_pitch;
	}

	return Measure ( images );
}

vector< pair <IMetricPlugin::ID, float> > CBFMPlugin::CalculateAverage ( bool visualize )
{
	vector< pair <IMetricPlugin::ID, float> > res;
	
	res.push_back ( make_pair ( m_bfm, m_b_error / m_frames_num ) );
	return res;
}

/*
* Private methods
*/

void CBFMPlugin::SetRGB ( BYTE * img, int x, int y, BYTE r, BYTE g, BYTE b )
{
	int ofs = ( y * m_width + x ) * 3;

	img [ ofs + 2 ] = r;
	img [ ofs + 1 ] = g;
	img [ ofs + 0 ] = b;
}

/*
* Config
*/

vector< pair <IMetricPlugin::ID, pair< vector <int>, wstring> > > CBFMPlugin::MapIDToFrame ( bool visualize )
{
	vector< pair <IMetricPlugin::ID, pair<vector <int>, wstring> > > res;

	/*
	* frames - vector of numbers of frames that given ID corresponds to. 
	* Here we have two IDs, both of which correspond to both frames 
	*/
	vector<int> frames;
	frames.push_back ( 0 );
	res.push_back ( make_pair ( m_bfm, make_pair ( frames, L"MSU BFM" ) ) );
	return res;
}

int CBFMPlugin::GetVideoNum ( bool visualize )
{
	return 1;
}

vector <IMetricPlugin::COLORCOMPONENT> CBFMPlugin::GetSupportedColorcomponents ()
{
	vector <IMetricPlugin::COLORCOMPONENT> cc;
	cc.push_back(IMetricPlugin::YYUV);
	return cc;
}

wstring CBFMPlugin::GetName ()
{
	return L"bfm";
}

wstring CBFMPlugin::GetUnit ()
{
	return L"";
}

int CBFMPlugin::GetWidthMultiply ()
{
	return 1;
}

int CBFMPlugin::GetHeightMultiply ()
{
	return 1;
}

wstring	CBFMPlugin::GetMetrInfoURL ()
{
	return L"http://compression.ru/video/quality_measure/metric_plugins/bfm_en.htm";
}

bool CBFMPlugin::GetMetrIncline ()
{
	return true;
}

wstring	CBFMPlugin::GetInterfaceName ()
{
	return L"MSU Brightness Flicking Metric v1.1";
}

bool CBFMPlugin::IsConfigurable ()
{
	return false;
}

bool CBFMPlugin::Config ( const wstring &configuration_string )
{
	return true;
}

void CBFMPlugin::GetDefaultConfiguration ( wstring &def_config )
{
	return;
}

//EOF//////////////////////////////////////////////////////////////////////////
